import { CubeSymbols, type ToString } from './CubeSymbols';
import type { CubeValidator } from './CubeValidator';
import type { ErrorReporter } from './ErrorReporter';
export type SegmentDefinition = {
    type: string;
    sql: Function;
    primaryKey?: true;
    ownedByCube: boolean;
    fieldType?: string;
    multiStage?: boolean;
};
export type DimensionDefinition = {
    type: string;
    sql: Function;
    primaryKey?: true;
    ownedByCube: boolean;
    fieldType?: string;
    multiStage?: boolean;
    shiftInterval?: string;
};
export type TimeShiftDefinition = {
    timeDimension?: (...args: Array<unknown>) => ToString;
    interval: string;
    type: 'next' | 'prior';
};
export type TimeShiftDefinitionReference = {
    timeDimension?: string;
    interval: string;
    type: 'next' | 'prior';
};
export type MeasureDefinition = {
    type: string;
    sql: Function;
    ownedByCube: boolean;
    rollingWindow?: any;
    filters?: any;
    primaryKey?: true;
    drillFilters?: any;
    multiStage?: boolean;
    groupBy?: (...args: Array<unknown>) => Array<ToString>;
    reduceBy?: (...args: Array<unknown>) => Array<ToString>;
    addGroupBy?: (...args: Array<unknown>) => Array<ToString>;
    timeShift?: TimeShiftDefinition[];
    groupByReferences?: string[];
    reduceByReferences?: string[];
    addGroupByReferences?: string[];
    timeShiftReferences?: TimeShiftDefinitionReference[];
    patchedFrom?: {
        cubeName: string;
        name: string;
    };
};
export type PreAggregationFilters = {
    dataSources?: string[];
    cubes?: string[];
    preAggregationIds?: string[];
    scheduled?: boolean;
};
export type EveryInterval = string;
type EveryCronInterval = string;
type EveryCronTimeZone = string;
export type CubeRefreshKeySqlVariant = {
    sql: () => string;
    every?: EveryInterval;
};
export type CubeRefreshKeyEveryVariant = {
    every: EveryInterval | EveryCronInterval;
    timezone?: EveryCronTimeZone;
    incremental?: boolean;
    updateWindow?: EveryInterval;
};
export type CubeRefreshKeyImmutableVariant = {
    immutable: true;
};
export type CubeRefreshKey = CubeRefreshKeySqlVariant | CubeRefreshKeyEveryVariant | CubeRefreshKeyImmutableVariant;
export type PreAggregationDefinition = {
    type: 'autoRollup' | 'originalSql' | 'rollupJoin' | 'rollupLambda' | 'rollup';
    allowNonStrictDateRangeMatch?: boolean;
    useOriginalSqlPreAggregations?: boolean;
    timeDimensionReference?: () => ToString;
    granularity: string;
    timeDimensionReferences: Array<{
        dimension: () => ToString;
        granularity: string;
    }>;
    dimensionReferences: () => Array<ToString>;
    segmentReferences: () => Array<ToString>;
    measureReferences: () => Array<ToString>;
    rollupReferences: () => Array<ToString>;
    indexes?: Record<string, any>;
    refreshKey?: CubeRefreshKey;
    scheduledRefresh: boolean;
    external: boolean;
};
export type PreAggregationDefinitions = Record<string, PreAggregationDefinition>;
export type PreAggregationTimeDimensionReference = {
    dimension: string;
    granularity: string;
};
export type PreAggregationReferences = {
    allowNonStrictDateRangeMatch?: boolean;
    dimensions: Array<string>;
    measures: Array<string>;
    timeDimensions: Array<PreAggregationTimeDimensionReference>;
    rollups: Array<string>;
    multipliedMeasures?: Array<string>;
};
export type PreAggregationInfo = {
    id: string;
    preAggregationName: string;
    preAggregation: unknown;
    cube: string;
    references: PreAggregationReferences;
    refreshKey: unknown;
    indexesReferences: unknown;
};
export declare class CubeEvaluator extends CubeSymbols {
    protected readonly cubeValidator: CubeValidator;
    evaluatedCubes: Record<string, any>;
    primaryKeys: Record<string, any>;
    byFileName: Record<string, any>;
    private isRbacEnabledCache;
    constructor(cubeValidator: CubeValidator);
    compile(cubes: any[], errorReporter: ErrorReporter): void;
    protected prepareCube(cube: any, errorReporter: ErrorReporter): any;
    private allMembersOrList;
    private prepareAccessPolicy;
    private prepareFolders;
    private prepareHierarchies;
    private evaluateMultiStageReferences;
    protected prepareJoins(cube: any, _errorReporter: ErrorReporter): void;
    protected preparePreAggregations(cube: any, errorReporter: ErrorReporter): void;
    protected prepareMembers(members: any, cube: any, errorReporter: ErrorReporter): void;
    cubesByFileName(fileName: any): any;
    timeDimensionPathsForCube(cube: any): string[];
    measuresForCube(cube: any): any;
    timeDimensionsForCube(cube: any): any[];
    preAggregationsForCube(path: string): Record<string, PreAggregationDefinition>;
    preAggregationsForCubeAsArray(path: string): {
        name: string;
    }[];
    /**
     * Returns pre-aggregations filtered by the specified selector.
     */
    preAggregations(filter: PreAggregationFilters): Array<PreAggregationInfo>;
    scheduledPreAggregations(): Array<PreAggregationInfo>;
    cubeNames(): string[];
    isMeasure(measurePath: string | string[]): boolean;
    isDimension(path: string | string[]): boolean;
    isSegment(path: string | string[]): boolean;
    measureByPath(measurePath: string): MeasureDefinition;
    dimensionByPath(dimensionPath: string): DimensionDefinition;
    segmentByPath(segmentPath: string): SegmentDefinition;
    cubeExists(cube: any): boolean;
    cubeFromPath(path: string): any;
    cubeNameFromPath(path: string): string;
    isInstanceOfType(type: 'measures' | 'dimensions' | 'segments', path: string | string[]): boolean;
    byPathAnyType(path: string | string[]): any;
    byPath(type: 'measures' | 'dimensions' | 'segments' | 'preAggregations', path: string | string[]): any;
    parsePath(type: 'measures' | 'dimensions' | 'segments' | 'preAggregations', path: string): string[];
    isRbacEnabledForCube(cube: any): boolean;
    isRbacEnabled(): boolean;
    parsePathAnyType(path: string): string[];
    collectUsedCubeReferences(cube: any, sqlFn: any): {
        cubeReferencesUsed: string[];
        pathReferencesUsed: string[][];
        evaluatedSql: unknown;
    };
    evaluatePreAggregationReferences(cube: string, aggregation: PreAggregationDefinition): PreAggregationReferences;
}
export {};
//# sourceMappingURL=CubeEvaluator.d.ts.map