"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CubePreAggregationConverter = void 0;
const parser_1 = require("@babel/parser");
const t = __importStar(require("@babel/types"));
const yaml_1 = __importStar(require("yaml"));
const UserError_1 = require("../UserError");
class CubePreAggregationConverter {
    preAggregationDefinition;
    constructor(preAggregationDefinition) {
        this.preAggregationDefinition = preAggregationDefinition;
    }
    convert(astByCubeName) {
        const { cubeName } = this.preAggregationDefinition;
        const cubeDefSet = astByCubeName[cubeName];
        if ('ast' in cubeDefSet) {
            this.convertJS(cubeDefSet);
        }
        else {
            this.convertYaml(cubeDefSet);
        }
    }
    convertJS(cubeDefSet) {
        const { preAggregationName, code } = this.preAggregationDefinition;
        const { cubeDefinition } = cubeDefSet;
        let preAggregationNode = null;
        const preAggregationAst = (0, parser_1.parse)(`(${code})`);
        if (t.isExpressionStatement(preAggregationAst.program.body[0])) {
            const [statement] = preAggregationAst.program.body;
            if (t.isObjectExpression(statement.expression)) {
                preAggregationNode = statement.expression;
            }
        }
        if (preAggregationNode === null) {
            throw new Error('Pre-aggregation definition is malformed');
        }
        let anchor = null;
        cubeDefinition.properties.forEach((prop) => {
            if (t.isObjectProperty(prop) && t.isIdentifier(prop.key)) {
                if (prop.key.name === 'preAggregations' && t.isObjectExpression(prop.value)) {
                    anchor = prop.value;
                    prop.value.properties.forEach((p) => {
                        if (t.isObjectProperty(p) && t.isIdentifier(p.key)) {
                            if (p.key.name === preAggregationName) {
                                throw new UserError_1.UserError(`Pre-aggregation '${preAggregationName}' is already defined`);
                            }
                        }
                    });
                }
            }
        });
        if (anchor === null) {
            cubeDefinition.properties.push(t.objectProperty(t.identifier('preAggregations'), t.objectExpression([t.objectProperty(t.identifier(preAggregationName), preAggregationNode)])));
        }
        else {
            anchor.properties.push(t.objectProperty(t.identifier(preAggregationName), preAggregationNode));
        }
    }
    convertYaml(cubeDefSet) {
        const { preAggregationName, code } = this.preAggregationDefinition;
        const { cubeDefinition } = cubeDefSet;
        const preAggDoc = yaml_1.default.parseDocument(code);
        const preAggNode = preAggDoc.contents;
        if (!preAggNode || !(0, yaml_1.isMap)(preAggNode)) {
            throw new UserError_1.UserError('Pre-aggregation YAML must be a map/object');
        }
        const preAggsPair = cubeDefinition.items.find((pair) => (0, yaml_1.isScalar)(pair.key) && (pair.key.value === 'pre_aggregations' || pair.key.value === 'preAggregations'));
        if (preAggsPair) {
            const seq = preAggsPair.value;
            if (!yaml_1.default.isSeq(seq)) {
                throw new UserError_1.UserError('\'pre_aggregations\' must be a sequence');
            }
            const exists = seq.items.some(item => {
                if ((0, yaml_1.isMap)(item)) {
                    const namePair = item.items.find((pair) => (0, yaml_1.isScalar)(pair.key) && pair.key.value === 'name');
                    return namePair && (0, yaml_1.isScalar)(namePair.value) && namePair.value.value === preAggregationName;
                }
                return false;
            });
            if (exists) {
                throw new UserError_1.UserError(`Pre-aggregation '${preAggregationName}' is already defined`);
            }
            seq.items.push(preAggNode);
        }
        else {
            const newSeq = new yaml_1.YAMLSeq();
            newSeq.items.push(preAggNode);
            cubeDefinition.items.push(new yaml_1.Pair(new yaml_1.Scalar('pre_aggregations'), newSeq));
        }
    }
}
exports.CubePreAggregationConverter = CubePreAggregationConverter;
//# sourceMappingURL=CubePreAggregationConverter.js.map